@@ -1,9 +1,17 @@
 # flake8: noqa
 from __future__ import absolute_import
+from .constants import STRICTNESS
 from .filterset import FilterSet
 from .filters import *
 
-__version__ = '0.15.3'
+# We make the `rest_framework` module available without an additional import.
+#   If DRF is not installed we simply set None.
+try:
+    from . import rest_framework
+except ImportError:
+    rest_framework = None
+
+__version__ = '1.0.1'
 
 
 def parse_version(version):
